﻿/******************************************************************************

Erstellung der BSS-Tabellen für die Job Verarbeitung für BSS 3.5

Letzte Änderungen:	08.06.2018
$Revision: 933 $
$Date: 2018-06-08 13:28:25 +0200 (Fr, 08 Jun 2018) $
$Header: http://svn.balvi.intern/DB-Skripte/trunk/BALVI%20iP/CREATE_BSS_TABLES_3_5.SQL 933 2018-06-08 11:28:25Z oj@BALVI.INTERN $

******************************************************************************/

DECLARE 
  vCnt INT;
  vIDX_TBS VARCHAR2(50) := NULL; 
  vUI_TBS VARCHAR2(60) := NULL; 
        
  FUNCTION FieldExists(pTable_name IN VARCHAR2, pColumn_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tab_columns
    WHERE table_name = pTable_name
      AND column_name = pColumn_name;
      
    RETURN vCnt > 0; 
  
  END;

  FUNCTION TableExists(pTable_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tables
    WHERE table_name = pTable_name;
      
    RETURN vCnt > 0; 
  
  END;
  
  FUNCTION IdxExists(pIndex_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_indexes
    WHERE index_name = pIndex_Name;
      
    RETURN vCnt > 0; 
  
  END;  

  PROCEDURE COMMENT_COL(pTab_Col IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON COLUMN ' || pTab_Col || ' IS ''' || pComment || '''';  
  END;
  
  PROCEDURE COMMENT_TAB(pTab IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON TABLE ' || pTab || ' IS ''' || pComment || '''';  
  END;
          
  PROCEDURE CrtIdx(pSQL IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE pSQL || vIDX_TBS;
  END; 

  
BEGIN

  -- Index-Tablespace merken
  SELECT tablespace_name INTO vIDX_TBS
  FROM user_indexes
  WHERE index_name = 'PK_D0101_ID';
  
  IF vIDX_TBS IS NOT NULL THEN
    vIDX_TBS := ' TABLESPACE ' || vIDX_TBS;
    vUI_TBS := ' USING INDEX ' || vIDX_TBS;
  END IF;
  
  
  IF NOT TableExists('JOB_QUEUE_ENTRIES') THEN  
        EXECUTE IMMEDIATE '
      CREATE TABLE JOB_QUEUE_ENTRIES
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           JOB_DEF_ID VARCHAR2(32) NOT NULL ,
           POS NUMBER(10) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           JOB_PARAMS_JSON CLOB,
           
		   CONSTRAINT PK_JOB_QUEUE_ENT$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE,
		   CONSTRAINT FK_JOB_QUEUE_ENT$JOB_DEF_ID FOREIGN KEY (JOB_DEF_ID) REFERENCES JOB_DEF (ID)
      )';
	
	COMMENT_TAB('JOB_QUEUE_ENTRIES', 'Job Queue persistieren und wieder herstellen, Jira BSS-184');
	
	COMMENT_COL('JOB_QUEUE_ENTRIES.ID', 'Primärschlüssel GUID');
	COMMENT_COL('JOB_QUEUE_ENTRIES.JOB_DEF_ID', 'Verweis auf den Job');
	COMMENT_COL('JOB_QUEUE_ENTRIES.POS', 'Reihenfolge in der Queue');
	COMMENT_COL('JOB_QUEUE_ENTRIES.INS_DATE', 'Wann hinzugefügt (rein informativ)');
	COMMENT_COL('JOB_QUEUE_ENTRIES.JOB_PARAMS_JSON', 'Parameter für den auszuführenden Job');
	
  END IF;

  
  IF NOT TableExists('JOB_HISTORY_ENTRIES') THEN  
        EXECUTE IMMEDIATE '
      CREATE TABLE JOB_HISTORY_ENTRIES
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           JOB_DEF_ID VARCHAR2(32) NOT NULL ,
           STARTED_DATE DATE NULL ,
           FINISHED_DATE DATE NULL ,
           STATUS VARCHAR2(30) NULL ,
           ITEM_COUNT NUMBER(38) DEFAULT 0 NULL ,
		   EXEC_INDEX NUMBER(10) DEFAULT 0 NULL ,
		   ERROR CLOB ,
		   JOB_PARAMS_JSON CLOB ,
           
		   CONSTRAINT PK_JOB_HISTORY_ENT$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE,
		   CONSTRAINT FK_JOB_HISTORY_ENT$JOB_DEF_ID FOREIGN KEY (JOB_DEF_ID) REFERENCES JOB_DEF (ID)
      )';
	
	COMMENT_TAB('JOB_HISTORY_ENTRIES', 'Anzeige der letzten Durchläufe pro Job, Jira BSS-185');
	
	COMMENT_COL('JOB_HISTORY_ENTRIES.ID', 'Primärschlüssel GUID');
	COMMENT_COL('JOB_HISTORY_ENTRIES.JOB_DEF_ID', 'Verweis auf den Job');
	COMMENT_COL('JOB_HISTORY_ENTRIES.STARTED_DATE', 'Startzeitpunkt des Jobs');
	COMMENT_COL('JOB_HISTORY_ENTRIES.FINISHED_DATE', 'Endzeitpunkt des Jobs');
	COMMENT_COL('JOB_HISTORY_ENTRIES.STATUS', 'Status des Jobdurchlaufs');
	COMMENT_COL('JOB_HISTORY_ENTRIES.ITEM_COUNT', 'Verarbeitete Datensätze');
	COMMENT_COL('JOB_HISTORY_ENTRIES.EXEC_INDEX', 'Index bei der Ausführung');
	COMMENT_COL('JOB_HISTORY_ENTRIES.ERROR', 'Fehlermeldung');
	COMMENT_COL('JOB_HISTORY_ENTRIES.JOB_PARAMS_JSON', 'Parameter für den auszuführenden Job');
  END IF;
  
 
END;
/

