﻿/******************************************************************************

Erstellung der ETL-Tabellen

Letzte Änderungen:	23.03.2016
$Revision: 898 $
$Date: 2018-02-15 14:30:01 +0100 (Do, 15 Feb 2018) $
$Header: http://svn.balvi.intern/DB-Skripte/trunk/BALVI%20iP/CREATE_ETL_TABLES.SQL 898 2018-02-15 13:30:01Z fb@BALVI.INTERN $

******************************************************************************/

DECLARE 
  vCnt INT;
  vIDX_TBS VARCHAR2(50) := NULL; 
  vUI_TBS VARCHAR2(60) := NULL; 
        
  FUNCTION FieldExists(pTable_name IN VARCHAR2, pColumn_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tab_columns
    WHERE table_name = pTable_name
      AND column_name = pColumn_name;
      
    RETURN vCnt > 0; 
  
  END;

  FUNCTION TableExists(pTable_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tables
    WHERE table_name = pTable_name;
      
    RETURN vCnt > 0; 
  
  END;
  
  FUNCTION IdxExists(pIndex_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_indexes
    WHERE index_name = pIndex_Name;
      
    RETURN vCnt > 0; 
  
  END;  

  PROCEDURE COMMENT_COL(pTab_Col IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON COLUMN ' || pTab_Col || ' IS ''' || pComment || '''';  
  END;
  
  PROCEDURE COMMENT_TAB(pTab IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON TABLE ' || pTab || ' IS ''' || pComment || '''';  
  END;
          
  PROCEDURE CrtIdx(pSQL IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE pSQL || vIDX_TBS;
  END; 

  
BEGIN

  -- Index-Tablespace merken
  SELECT tablespace_name INTO vIDX_TBS
  FROM user_indexes
  WHERE index_name = 'PK_D0101_ID';
  
  IF vIDX_TBS IS NOT NULL THEN
    vIDX_TBS := ' TABLESPACE ' || vIDX_TBS;
    vUI_TBS := ' USING INDEX ' || vIDX_TBS;
  END IF;
  
  
  IF NOT TableExists('BSS_ETL_JOBS') THEN  
        EXECUTE IMMEDIATE '
      CREATE TABLE BSS_ETL_JOBS
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           CONTEXT VARCHAR2(64) NOT NULL ,
           VERSION VARCHAR2(32) NOT NULL ,
           CODE VARCHAR2(128) NOT NULL ,
           DESCRIPTION VARCHAR2(2000) NULL ,
           FILE_REPOS_ETL_ID VARCHAR2(32) NOT NULL ,
		   ROOT_JOB_NAME VARCHAR2(2000) NOT NULL ,
           CREATED_DATE DATE DEFAULT SYSDATE NOT NULL ,
           UPDATE_DATE DATE DEFAULT SYSDATE NOT NULL ,
           
		   CONSTRAINT PK_BSS_ETL_JOBS$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE,
		   CONSTRAINT FK_BSS_ETL_JOBS$T_FILE_REP_ID FOREIGN KEY (FILE_REPOS_ETL_ID) REFERENCES TEXT_FILE_REPOSITORY (ID)
      )';
	
	CrtIdx('CREATE UNIQUE INDEX UQ_BSS_ETL_JOBS$CTX_VER_COD ON BSS_ETL_JOBS (CONTEXT,VERSION,UPPER(CODE))');
	
	COMMENT_TAB('BSS_ETL_JOBS', 'Konfiguration der ETL-Jobs');
	
	COMMENT_COL('BSS_ETL_JOBS.ID', 'Primärschlüssel GUID');
	COMMENT_COL('BSS_ETL_JOBS.CONTEXT', 'Code des Verfahrens zu dem der ETL Job gehört');
	COMMENT_COL('BSS_ETL_JOBS.VERSION', 'Version des ETL-Jobs');
	COMMENT_COL('BSS_ETL_JOBS.CODE', 'Code der ETL-Jobs, UPPER-Unique mit Context und Version');
	COMMENT_COL('BSS_ETL_JOBS.DESCRIPTION', 'Beschreibung des ETL-Jobs');
	COMMENT_COL('BSS_ETL_JOBS.FILE_REPOS_ETL_ID', 'Fremdschlüssel auf Zip Archiv des Kettle Repo (beinhaltet Job und Transformationen) in TEXT_FILE_REPOSITORY');
	COMMENT_COL('BSS_ETL_JOBS.ROOT_JOB_NAME', 'Name des Jobs aus Kettle-Zip, der gestartet werden soll - ggf. inkl. Pfadangabe.');
	COMMENT_COL('BSS_ETL_JOBS.CREATED_DATE', 'Wann hinzugefügt (rein Informativ)');
	COMMENT_COL('BSS_ETL_JOBS.UPDATE_DATE', 'Wann aktualisiert (rein Informativ)');
	
  END IF;

  
  IF NOT TableExists('BSS_ENDPOINTS') THEN  
        EXECUTE IMMEDIATE '
      CREATE TABLE BSS_ENDPOINTS
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           CONTEXT VARCHAR2(64) NOT NULL ,
           ENDPOINT_TYPE NUMBER(2) DEFAULT 0 NOT NULL ,
           SERVICE_NAME VARCHAR2(255) NOT NULL ,
           DESCRIPTION VARCHAR2(2000) NULL ,
           SERVER_VERSIONS VARCHAR2(64) DEFAULT ''* - *'' NOT NULL ,
		   FILE_REPOS_XSD_ID VARCHAR2(32) DEFAULT ''-1'' NOT NULL ,
		   FILE_REPOS_WSDL_ID VARCHAR2(32) DEFAULT ''-1'' NOT NULL ,
		   FILE_REPOS_GROOVY_ID VARCHAR2(32) DEFAULT ''-1'' NOT NULL ,
		   BSS_ETL_JOBS_ID VARCHAR2(32) NOT NULL ,
           CREATED_DATE DATE DEFAULT SYSDATE NOT NULL ,
           UPDATE_DATE DATE DEFAULT SYSDATE NOT NULL ,
           
		   CONSTRAINT PK_BSS_ENDPOINTS$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE,
		   CONSTRAINT FK_BSS_ENDPOINTS$REPOS_XSD_ID FOREIGN KEY (FILE_REPOS_XSD_ID) REFERENCES TEXT_FILE_REPOSITORY (ID),
		   CONSTRAINT FK_BSS_ENDPOINTS$REPOS_WSDL_ID FOREIGN KEY (FILE_REPOS_WSDL_ID) REFERENCES TEXT_FILE_REPOSITORY (ID),
		   CONSTRAINT FK_BSS_ENDPOINTS$REPOS_GRV_ID FOREIGN KEY (FILE_REPOS_GROOVY_ID) REFERENCES TEXT_FILE_REPOSITORY (ID),
		   CONSTRAINT FK_BSS_ENDPOINTS$ETL_JOB_ID FOREIGN KEY (BSS_ETL_JOBS_ID) REFERENCES BSS_ETL_JOBS (ID)
      )';
	
	CrtIdx('CREATE UNIQUE INDEX UQ_BSS_ENDPOINTS$SERVICE_NAME ON BSS_ENDPOINTS (UPPER(SERVICE_NAME))');
	
	COMMENT_TAB('BSS_ENDPOINTS', 'Konfiguration der ETL-Jobs');
	
	COMMENT_COL('BSS_ENDPOINTS.ID', 'Primärschlüssel GUID');
	COMMENT_COL('BSS_ENDPOINTS.CONTEXT', 'Verfahren zu dem der Endpunkt gehört');
	COMMENT_COL('BSS_ENDPOINTS.ENDPOINT_TYPE', '0 - SOAP, 1 - http/REST');
	COMMENT_COL('BSS_ENDPOINTS.SERVICE_NAME', 'Name des Endpunkts -> daraus ergibt sich auch die URL: bei SOAP: soapws/{SERVICE_NAME}, bei http/REST: rest/{SERVICE_NAME}');
	COMMENT_COL('BSS_ENDPOINTS.DESCRIPTION', 'Beschreibung des Endpunktes');
	COMMENT_COL('BSS_ENDPOINTS.SERVER_VERSIONS', 'Unterstützung für entsprechende Server Versionen');
	COMMENT_COL('BSS_ENDPOINTS.FILE_REPOS_XSD_ID', 'ID der FileResource im TEXT_FILE_REPO welche XSD für SOAP Endpunkt enthält -> Daraus wird automatisch eine WSDL erzeugt falls keine WSDL explizit angegeben wurde');
	COMMENT_COL('BSS_ENDPOINTS.FILE_REPOS_WSDL_ID', 'ID der FileResource im TEXT_FILE_REPO welche WSDL beinhaltet die für SOAP Endpunkt verwendet werden soll');
	COMMENT_COL('BSS_ENDPOINTS.FILE_REPOS_GROOVY_ID', 'ID der FileResource im TEXT_FILE_REPO zu Groovy Script welches die Endpunkt Implementierung beinhaltet');
	COMMENT_COL('BSS_ENDPOINTS.BSS_ETL_JOBS_ID', 'ETL-Job der Standardmäßig ausgeführt werden soll bei Endpunkt Aufruf');
	COMMENT_COL('BSS_ENDPOINTS.CREATED_DATE', 'Wann hinzugefügt (rein Informativ)');
	COMMENT_COL('BSS_ENDPOINTS.UPDATE_DATE', 'Wann aktualisiert (rein Informativ)');
	
  END IF;
  
  IF NOT TableExists('IP2_ID_MAPPING') THEN  
        EXECUTE IMMEDIATE '
			  CREATE TABLE IP2_ID_MAPPING
				( id varchar2(32) default sys_guid(),
				  ip1_id varchar2(32) NOT NULL,
				  ip2_id varchar2(32) NOT NULL,
				  ip1_table varchar2(64),
				  ip2_context varchar2(255),
				  
				  CONSTRAINT PK_IP2_ID_MAPPING_ID PRIMARY KEY (id) NOT DEFERRABLE Initially IMMEDIATE
				)';
	
	COMMENT_TAB('IP2_ID_MAPPING', 'Konfiguration der ETL-Jobs');
	
	COMMENT_COL('IP2_ID_MAPPING.ID', 'Primärschlüssel GUID');
	COMMENT_COL('IP2_ID_MAPPING.IP1_ID', 'ID in iP1');
	COMMENT_COL('IP2_ID_MAPPING.IP2_ID', 'ID in iP2');
	COMMENT_COL('IP2_ID_MAPPING.IP1_TABLE', 'Tabelle aus der/in die die iP1-ID stammt. Kann ggf. auch ein einzelnes Feld bezeichnen, wenn ein iP2-Objekt auf ein Feld in iP1 abgebildet wird, z.B. D010801.D0301_ID');
	COMMENT_COL('IP2_ID_MAPPING.IP2_CONTEXT', 'Context, aus dem die iP2-ID stammt. Formatrichtlinie: app-name.objekt:typ z.B. lm-kontrolle-app.termin:nachkontrolle');
	
  END IF;
  
  IF NOT FieldExists('IP2_ID_MAPPING','LAST_SYNC') THEN
    EXECUTE IMMEDIATE '
	  ALTER TABLE IP2_ID_MAPPING
	    ADD LAST_SYNC DATE DEFAULT TO_DATE(''31.12.1920'',''dd.mm.yyyy'') NOT NULL
    ';
	COMMENT_COL('IP2_ID_MAPPING.LAST_SYNC', 'Datum des letzten Abgleichs nach iP2');
  END IF;
  
  IF NOT IdxExists('IDX_IP2_MAP_ID$CTX$LS') THEN
    CrtIdx('CREATE INDEX IDX_IP2_MAP_ID$CTX$LS ON IP2_ID_MAPPING (IP1_ID,IP2_CONTEXT,LAST_SYNC)');
  END IF;

  IF NOT IdxExists('IDX_IP2_MAP_TAB$ID$CTX') THEN
    CrtIdx('CREATE INDEX IDX_IP2_MAP_TAB$ID$CTX ON IP2_ID_MAPPING (IP1_TABLE,IP1_ID,IP2_CONTEXT)');
  END IF;

  IF NOT IdxExists('IDX_IP2_MAP_TAB$IP2_ID$CTX') THEN
    CrtIdx('CREATE INDEX IDX_IP2_MAP_TAB$IP2_ID$CTX ON IP2_ID_MAPPING (IP2_ID,IP2_CONTEXT,IP1_TABLE)');
  END IF;

END;
/

