CREATE OR REPLACE TRIGGER TRG_D0101_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE ON VIS_D0101 
FOR EACH ROW
DECLARE
  sFB VARCHAR2(32);
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	DELETE FROM WKD_D0101_MOBIL WHERE ID = sD0101_ID;
	
  END CASE;
  
  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  IF NOT DELETING THEN
    /* SAVE FAST INSERT OR UPDATE */
    SELECT COUNT(*) INTO nROWCNT FROM WKD_D0101_MOBIL WHERE ID = sD0101_ID;
    IF nROWCNT = 0  THEN
         INSERT INTO WKD_D0101_MOBIL (ID,D0101_UPDATE) VALUES (sD0101_ID, dUPD_DATE);
     ELSE
        UPDATE WKD_D0101_MOBIL SET D0101_UPDATE = dUPD_DATE
        WHERE ID = sD0101_ID AND D0101_UPDATE < dUPD_DATE;
     END IF;
  END IF;
  
END;
/

CREATE OR REPLACE TRIGGER TRG_D010101_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010101
FOR EACH ROW
WHEN (   NEW.FB LIKE '%H%' OR NEW.FB LIKE '%U%' OR NEW.FB LIKE '%T%' OR NEW.FB='%'
      OR OLD.FB LIKE '%H%' OR OLD.FB LIKE '%U%' OR OLD.FB LIKE '%T%' OR OLD.FB='%')
DECLARE
  sFB VARCHAR2(32);
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN
  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;
  
  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND :NEW.A010108_SKZ = 2 AND (:NEW.FB LIKE '%H%' OR :NEW.FB='%')
                OR :OLD.A010108_SKZ = 2 AND (:OLD.FB LIKE '%H%' OR :OLD.FB='%')
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM   (kein festes SKZ)  */
  IF INSERTING AND (:NEW.FB LIKE '%T%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%T%' OR :OLD.FB='%')
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM        */
  IF INSERTING AND :NEW.A010108_SKZ = 3 AND (:NEW.FB LIKE '%U%' OR :NEW.FB='%')
                OR :OLD.A010108_SKZ = 3 AND (:OLD.FB LIKE '%U%' OR :OLD.FB='%')
  THEN nFM_UPD:=1;
  ELSE nFM_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 AND dUPD_DATE>recUPD_DATA.LM_KTR THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
      recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 AND dUPD_DATE>recUPD_DATA.TS_KTR THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
      recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_UPD=1 AND dUPD_DATE>recUPD_DATA.FM_KTR THEN
      recUPD_DATA.FM_KTR:=dUPD_DATE;
      recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
      recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
      recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
      recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

CREATE OR REPLACE TRIGGER TRG_D010110_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010110
FOR EACH ROW
WHEN (   NEW.A01080201_SID IN (5394001, 5395001, 7424001, 7425001, 7426001, 19748001, 19750001, 19751001, 19752001, 19754001, 19755001, 19756001, 22214001, 12519001, 19596001 )
      OR OLD.A01080201_SID IN (5394001, 5395001, 7424001, 7425001, 7426001, 19748001, 19750001, 19751001, 19752001, 19754001, 19755001, 19756001, 22214001, 12519001, 19596001 )
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;

BEGIN
  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;
  
  /* FACHLOGIK */
  /* keine. Fhrt immer zum Update des Betriebsdatums */
  
  /* SAVE FAST INSERT OR UPDATE */
  SELECT COUNT(*) INTO nROWCNT FROM WKD_D0101_MOBIL WHERE ID = sD0101_ID;
  IF nROWCNT = 0  THEN
       INSERT INTO WKD_D0101_MOBIL (ID,D0101_UPDATE) VALUES (sD0101_ID, dUPD_DATE);
   ELSE
      UPDATE WKD_D0101_MOBIL SET D0101_UPDATE = dUPD_DATE
      WHERE ID = sD0101_ID AND D0101_UPDATE < dUPD_DATE;
   END IF;

END;
/


/* Hier noch Klrungsbedarf ob beide Personaltabellen mit allen Mglichkeiten berwacht werden mssen */
/* ACHTUNG: weder INSERT noch DELETE werden hier abgefangen, weil man in den Fllen an die d0101_id nicht herankommt */
/* weil der dazu gehrige d010200 Satz noch nicht existiert bzw. schon gelscht ist! */
/* Sollte auch im UPDATE Fall kein d010200 Satz vorhanden sein (darf eigentlich nicht) wird unten die Exception gefangen */
/*
   22.04.2013: da Personen in mehreren Personaleintrgen verschiedener Betriebe vorkommen knnen hier Schleife eingebaut
*/
CREATE OR REPLACE TRIGGER TRG_D0102_MOBIL
AFTER UPDATE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D0102
FOR EACH ROW
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;

BEGIN

  CASE WHEN UPDATING ('UPD_DATE') THEN
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	dUPD_DATE := SYSDATE;

  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  /* FACHLOGIK */
  /* keine. Fhrt immer zum Update des Betriebsdatums */
  
  
  FOR recBetrieb IN (
    SELECT D0101_ID FROM VIS_D010200 D010200 WHERE D010200.deleted>-2 AND D010200.D0102_ID=:OLD.ID
  )
  
  LOOP
  
  /* SAVE FAST INSERT OR UPDATE */
  SELECT COUNT(*) INTO nROWCNT FROM WKD_D0101_MOBIL WHERE ID = recBetrieb.D0101_ID;
  IF nROWCNT = 0  THEN
       INSERT INTO WKD_D0101_MOBIL (ID,D0101_UPDATE) VALUES (recBetrieb.D0101_ID, dUPD_DATE);
   ELSE
      UPDATE WKD_D0101_MOBIL SET D0101_UPDATE = dUPD_DATE
      WHERE ID = recBetrieb.D0101_ID AND D0101_UPDATE < dUPD_DATE;
   END IF;
  
  END LOOP;
  
  EXCEPTION WHEN NO_DATA_FOUND THEN
      sD0101_ID:='-1';

END;
/

/* Hier noch Klrungsbedarf ob beide Personaltabellen mit allen Mglichkeiten berwacht werden mssen */
CREATE OR REPLACE TRIGGER TRG_D010200_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010200
FOR EACH ROW
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;

BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;
  
  
  /* FACHLOGIK */
  /* keine. Fhrt immer zum Update des Betriebsdatums */
  
  /* SAVE FAST INSERT OR UPDATE */
  SELECT COUNT(*) INTO nROWCNT FROM WKD_D0101_MOBIL WHERE ID = sD0101_ID;
  IF nROWCNT = 0  THEN
       INSERT INTO WKD_D0101_MOBIL (ID,D0101_UPDATE) VALUES (sD0101_ID, dUPD_DATE);
   ELSE
      UPDATE WKD_D0101_MOBIL SET D0101_UPDATE = dUPD_DATE
      WHERE ID = sD0101_ID AND D0101_UPDATE < dUPD_DATE;
   END IF;

END;
/

/* Kontrollbereiche */
CREATE OR REPLACE TRIGGER TRG_D0103_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D0103
FOR EACH ROW
WHEN (   NEW.FB LIKE '%H%' OR NEW.FB LIKE '%T%' OR NEW.FB LIKE '%U%' OR NEW.FB='%'
      OR OLD.FB LIKE '%H%' OR OLD.FB LIKE '%T%' OR OLD.FB LIKE '%U%' OR OLD.FB='%'
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  nROWCNT NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;
  
BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND (:NEW.FB LIKE '%H%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%H%' OR :OLD.FB='%')
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM       */
  IF INSERTING AND (:NEW.FB LIKE '%T%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%T%' OR :OLD.FB='%')
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM       */
  IF INSERTING AND (:NEW.FB LIKE '%U%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%U%' OR :OLD.FB='%')
  THEN nFM_UPD:=1;
  ELSE nFM_UPD:=0;
  END IF;
  
  /* Aktualisieren bzw. neu anlegen */
   BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
      recUPD_DATA.LM_PRB:=CASE WHEN recUPD_DATA.LM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_PRB END;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=CASE WHEN recUPD_DATA.TS_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_KTR END;
      recUPD_DATA.TS_PRB:=CASE WHEN recUPD_DATA.TS_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_PRB END;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
      recUPD_DATA.FM_PRB:=CASE WHEN recUPD_DATA.FM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_PRB END;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
      recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
      recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
      recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Nummern */
CREATE OR REPLACE TRIGGER TRG_D010102_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010102
FOR EACH ROW
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
  nA010145_SID NUMBER;
  nA010145_SID0 NUMBER;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN
  CASE WHEN INSERTING THEN 
    sD0101_ID := :NEW.D0101_ID;
    dUPD_DATE := :NEW.UPD_DATE;
                        
  WHEN UPDATING ('UPD_DATE') THEN
    sD0101_ID := :OLD.D0101_ID;
    dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
    sD0101_ID := :OLD.D0101_ID;
    dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
    sD0101_ID := :OLD.D0101_ID;
    dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
    sD0101_ID := :OLD.D0101_ID;
    dUPD_DATE := SYSDATE; 
    
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  /* EG Verordnung hat diverse Untereintrge, die logisch ber die a010145_sid0 zusammengehalten werden */
  BEGIN
    IF INSERTING 
    THEN SELECT A010145_SID0 INTO nA010145_SID0 FROM VIS_A010145 WHERE SID=:NEW.A010145_SID;
    ELSE SELECT A010145_SID0 INTO nA010145_SID0 FROM VIS_A010145 WHERE SID=:OLD.A010145_SID;
    END IF;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    nA010145_SID0:=-1;
  END;

  IF nA010145_SID0=1268001 OR (INSERTING AND :NEW.A010145_SID=1558001) OR :OLD.A010145_SID=1558001
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM        */
  IF INSERTING AND :NEW.A010145_SID IN (514001, 709001)
                OR :OLD.A010145_SID IN (514001, 709001)
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM        */
  BEGIN
    IF INSERTING 
    THEN SELECT sid INTO nA010145_SID FROM VIS_A010145 
          WHERE :NEW.A010145_SID=VIS_A010145.sid
            AND :NEW.A010145_SID IN (SELECT a010145.sid FROM vis_a010145 a010145 
                                     WHERE
                                          a010145.code LIKE '01110104%' AND a010145.SKZ=60
                                       OR a010145.code LIKE '01110105%' AND a010145.SKZ=60
                                       OR a010145.code = '01110107' AND a010145.SKZ=60
                                       OR a010145.code LIKE '011102%' AND a010145.SKZ=60
                                       OR a010145.code LIKE '011103%' AND a010145.SKZ=60
                                       OR a010145.code = '0102' AND a010145.SKZ=10
                                       OR a010145.code LIKE '021102%' AND a010145.SKZ=70
                                       OR a010145.code LIKE '021103%' AND a010145.SKZ=70
                                       OR a010145.code LIKE '021104%' AND a010145.SKZ=70
                                    );
    ELSE SELECT sid INTO nA010145_SID FROM VIS_A010145 
          WHERE :OLD.A010145_SID=VIS_A010145.sid
            AND :OLD.A010145_SID IN (SELECT a010145.sid FROM vis_a010145 a010145 
                                     WHERE
                                          a010145.code LIKE '01110104%' AND a010145.SKZ=60
                                       OR a010145.code LIKE '01110105%' AND a010145.SKZ=60
                                       OR a010145.code = '01110107' AND a010145.SKZ=60
                                       OR a010145.code LIKE '011102%' AND a010145.SKZ=60
                                       OR a010145.code LIKE '011103%' AND a010145.SKZ=60
                                       OR a010145.code = '0102' AND a010145.SKZ=10
                                       OR a010145.code LIKE '021102%' AND a010145.SKZ=70
                                       OR a010145.code LIKE '021103%' AND a010145.SKZ=70
                                       OR a010145.code LIKE '021104%' AND a010145.SKZ=70
                                    );
    END IF;
  EXCEPTION WHEN NO_DATA_FOUND THEN
    nA010145_SID:=-1;
  END;
  IF nA010145_SID<>-1
    THEN nFM_UPD:=1;
    ELSE nFM_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 THEN
      recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
      recUPD_DATA.LM_PRB:=CASE WHEN recUPD_DATA.LM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_PRB END;
    END IF;
    IF nTAM_UPD=1 THEN
      recUPD_DATA.TS_KTR:=CASE WHEN recUPD_DATA.TS_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_KTR END;
      recUPD_DATA.TS_PRB:=CASE WHEN recUPD_DATA.TS_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_PRB END;
    END IF;
    IF nFM_UPD=1 THEN
      recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
      recUPD_DATA.FM_PRB:=CASE WHEN recUPD_DATA.FM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_PRB END;
    END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
      recUPD_DATA.LM_KTR:=dUPD_DATE;
      recUPD_DATA.LM_PRB:=dUPD_DATE;
    END IF;
    IF nTAM_UPD=1 THEN
      recUPD_DATA.TS_KTR:=dUPD_DATE;
      recUPD_DATA.TS_PRB:=dUPD_DATE;
    END IF;
    IF nFM_UPD=1 THEN
      recUPD_DATA.FM_KTR:=dUPD_DATE;
      recUPD_DATA.FM_PRB:=dUPD_DATE;
    END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Termine */
/* Die SIDs entsprechen Code 01% und 02% mit PKS K und FB H bzw. % */
/* Kontrolle und Probe noch zu klren */
CREATE OR REPLACE TRIGGER TRG_D0301_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D0301
FOR EACH ROW
WHEN (   NEW.A0301_01_SID IN (26001,27001,31001,32001) --Plan- und Wiedervorlagetermine
     AND NEW.PKS IN ('K','S','Q') AND NEW.FB IN ('H','U','T')
      OR OLD.A0301_01_SID IN (26001,27001,31001,32001)
     AND OLD.PKS IN ('K','S','Q') AND OLD.FB IN ('H','U','T')
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN
  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND :NEW.FB='H'
                OR :OLD.FB='H'
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM        */
  IF INSERTING AND :NEW.FB='T'
                OR :OLD.FB='T'
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM        */
  IF INSERTING AND :NEW.FB='U'
                OR :OLD.FB='U'
  THEN nFM_UPD:=1;
  ELSE nFM_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
      recUPD_DATA.LM_PRB:=CASE WHEN recUPD_DATA.LM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_PRB END;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=CASE WHEN recUPD_DATA.TS_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_KTR END;
      recUPD_DATA.TS_PRB:=CASE WHEN recUPD_DATA.TS_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_PRB END;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
      recUPD_DATA.FM_PRB:=CASE WHEN recUPD_DATA.FM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_PRB END;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Fachbereiche */
CREATE OR REPLACE TRIGGER TRG_D010104_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010104
FOR EACH ROW
WHEN (   NEW.FB='H' OR NEW.FB='T' OR NEW.FB='U'
      OR OLD.FB='H' OR OLD.FB='T' OR OLD.FB='U'
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  nROWCNT NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN
  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND :NEW.FB='H'
                OR :OLD.FB='H'
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM        */
  IF INSERTING AND :NEW.FB='T'
                OR :OLD.FB='T'
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM        */
  IF INSERTING AND :NEW.FB='U'
                OR :OLD.FB='U'
  THEN nFM_UPD:=1;
  ELSE nFM_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
      recUPD_DATA.LM_PRB:=CASE WHEN recUPD_DATA.LM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_PRB END;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=CASE WHEN recUPD_DATA.TS_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_KTR END;
      recUPD_DATA.TS_PRB:=CASE WHEN recUPD_DATA.TS_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_PRB END;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
      recUPD_DATA.FM_PRB:=CASE WHEN recUPD_DATA.FM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_PRB END;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Ttigkeiten */
CREATE OR REPLACE TRIGGER TRG_D010801_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010801
FOR EACH ROW
WHEN ( (NEW.FB LIKE '%H%' OR NEW.FB LIKE '%U%' OR NEW.FB LIKE '%T%') AND NEW.PKS IN ('K','W','P','Q','N','S','Z','E') AND NEW.BESF IN ('B','%')
    OR (OLD.FB LIKE '%H%' OR OLD.FB LIKE '%U%' OR OLD.FB LIKE '%T%') AND OLD.PKS IN ('K','W','P','Q','N','S','Z','E') AND OLD.BESF IN ('B','%')
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
  nLM_KTR_UPD NUMBER;
  nLM_PRB_UPD NUMBER;
  nTAM_KTR_UPD NUMBER;
  nTAM_PRB_UPD NUMBER;
  nFM_KTR_UPD NUMBER;
  nFM_PRB_UPD NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN
  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  /* BESF Bedingung immer erfllt durch obiges WHEN, kann daher hier weggelassen werden */
  IF INSERTING AND :NEW.FB LIKE '%H%'AND :NEW.PKS IN ('K','W')
                OR :OLD.FB LIKE '%H%'AND :OLD.PKS IN ('K','W')
    THEN nLM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%H%'AND :NEW.PKS IN ('P')
                   OR :OLD.FB LIKE '%H%'AND :OLD.PKS IN ('P')
    THEN nLM_PRB_UPD:=1;
  ELSE nLM_KTR_UPD:=0; nLM_PRB_UPD:=0;
  END IF;
  
  /* TAM        */
  /* hier interessiert nur TAM */
  IF INSERTING AND :NEW.FB LIKE '%T%'AND :NEW.PKS IN ('S')
                OR :OLD.FB LIKE '%T%'AND :OLD.PKS IN ('S')
    THEN nTAM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%T%'AND :NEW.PKS IN ('Z','E')
                   OR :OLD.FB LIKE '%T%'AND :OLD.PKS IN ('Z','E')
    THEN nTAM_PRB_UPD:=1;
  ELSE nTAM_KTR_UPD:=0; nTAM_PRB_UPD:=0;
  END IF;
  
  /* FM        */
  /* hier interessiert nur FM */
  IF INSERTING AND :NEW.FB LIKE '%U%'AND :NEW.PKS IN ('Q')
                OR :OLD.FB LIKE '%U%'AND :OLD.PKS IN ('Q')
    THEN nFM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%U%'AND :NEW.PKS IN ('N')
                   OR :OLD.FB LIKE '%U%'AND :OLD.PKS IN ('N')
    THEN nFM_PRB_UPD:=1;
  ELSE nFM_KTR_UPD:=0; nFM_PRB_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.LM_KTR THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	END IF;
    IF nLM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.LM_PRB THEN
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.TS_KTR THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	END IF;
    IF nTAM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.TS_PRB THEN
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.FM_KTR THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	END IF;
    IF nFM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.FM_PRB THEN
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_KTR_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	END IF;
    IF nLM_PRB_UPD=1 THEN
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_KTR_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	END IF;
    IF nTAM_PRB_UPD=1 THEN
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
    IF nFM_KTR_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	END IF;
    IF nFM_PRB_UPD=1 THEN
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/


/* Risikobeurteilung erstmal nur LM und FM Kontrolle */
CREATE OR REPLACE TRIGGER TRG_D0115_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D0115
FOR EACH ROW
WHEN (   NEW.FB LIKE '%H%' OR NEW.FB LIKE '%U%' OR NEW.FB='%'
      OR OLD.FB LIKE '%H%' OR OLD.FB LIKE '%U%' OR OLD.FB='%'
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nLM_KTR_UPD NUMBER;
  nFM_KTR_UPD NUMBER;
  nROWCNT NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND (:NEW.FB LIKE '%H%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%H%' OR :OLD.FB='%')
  THEN nLM_KTR_UPD:=1;
  ELSE nLM_KTR_UPD:=0;
  END IF;
  
  /* FM       */
  IF INSERTING AND (:NEW.FB LIKE '%U%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%U%' OR :OLD.FB='%')
  THEN nFM_KTR_UPD:=1;
  ELSE nFM_KTR_UPD:=0;
  END IF;
  
/* Aktualisieren bzw. neu anlegen */
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_KTR_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
	END IF;
	IF nFM_KTR_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_KTR_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	END IF;
	IF nFM_KTR_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Manahmen */
/* Hier auch PKS aber auch % damit Manahmen direkt zum Betrieb auch anspringen */
CREATE OR REPLACE TRIGGER TRG_D010803_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010803
FOR EACH ROW
WHEN ( (NEW.FB LIKE '%H%' OR NEW.FB LIKE '%T%' OR NEW.FB LIKE '%U%' OR NEW.FB ='%') AND NEW.PKS IN ('K','P','E','Z','S','Q','N','%')
    OR (OLD.FB LIKE '%H%' OR OLD.FB LIKE '%T%' OR OLD.FB LIKE '%U%' OR OLD.FB ='%') AND OLD.PKS IN ('K','P','E','Z','S','Q','N','%')
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nLM_UPD NUMBER;
  nTAM_UPD NUMBER;
  nFM_UPD NUMBER;
  nROWCNT NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM        */
  IF INSERTING AND (:NEW.FB LIKE '%H%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%H%' OR :OLD.FB='%')
  THEN nLM_UPD:=1;
  ELSE nLM_UPD:=0;
  END IF;
  
  /* TAM       */
  IF INSERTING AND (:NEW.FB LIKE '%T%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%T%' OR :OLD.FB='%')
  THEN nTAM_UPD:=1;
  ELSE nTAM_UPD:=0;
  END IF;
  
  /* FM       */
  IF INSERTING AND (:NEW.FB LIKE '%U%' OR :NEW.FB='%')
                OR (:OLD.FB LIKE '%U%' OR :OLD.FB='%')
  THEN nFM_UPD:=1;
  ELSE nFM_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=CASE WHEN recUPD_DATA.LM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_KTR END;
      recUPD_DATA.LM_PRB:=CASE WHEN recUPD_DATA.LM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.LM_PRB END;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=CASE WHEN recUPD_DATA.TS_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_KTR END;
      recUPD_DATA.TS_PRB:=CASE WHEN recUPD_DATA.TS_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.TS_PRB END;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=CASE WHEN recUPD_DATA.FM_KTR<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_KTR END;
      recUPD_DATA.FM_PRB:=CASE WHEN recUPD_DATA.FM_PRB<dUPD_DATE THEN dUPD_DATE ELSE recUPD_DATA.FM_PRB END;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
	IF nFM_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/

/* Verste/Beanstandungen*/
CREATE OR REPLACE TRIGGER TRG_D010802_MOBIL
AFTER INSERT OR UPDATE OR DELETE OF UPD_DATE, DEL_DATE, DELETED ON VIS_D010802
FOR EACH ROW
WHEN ( (NEW.FB LIKE '%H%' OR NEW.FB LIKE '%T%' OR NEW.FB LIKE '%U%' OR NEW.FB ='%') AND NEW.PKS IN ('K','P','S','Z','E','Q','N')
    OR (OLD.FB LIKE '%H%' OR OLD.FB LIKE '%T%' OR OLD.FB LIKE '%U%' OR OLD.FB ='%') AND OLD.PKS IN ('K','P','S','Z','E','Q','N')
     )
DECLARE
  sD0101_ID VARCHAR2(32);
  dUPD_DATE DATE;
  nROWCNT NUMBER;
  nLM_KTR_UPD NUMBER;
  nLM_PRB_UPD NUMBER;
  nTAM_KTR_UPD NUMBER;
  nTAM_PRB_UPD NUMBER;
  nFM_KTR_UPD NUMBER;
  nFM_PRB_UPD NUMBER;
  recUPD_DATA WKD_D0101_MOBIL%ROWTYPE;

BEGIN

  CASE WHEN INSERTING THEN 
	sD0101_ID := :NEW.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;
	    	    		
  WHEN UPDATING ('UPD_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.UPD_DATE;

  WHEN UPDATING ('DEL_DATE') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN UPDATING ('DELETED') THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := :NEW.DEL_DATE;

  WHEN DELETING THEN
	sD0101_ID := :OLD.D0101_ID;
	dUPD_DATE := SYSDATE; 
	
  END CASE;

  IF dUPD_DATE IS NULL THEN     /* z.B. Entlschung */
    dUPD_DATE:=SYSDATE;
  END IF;

  
  /* FACHLOGIK */
  /* LM */
  IF INSERTING AND :NEW.FB LIKE '%H%'AND :NEW.PKS IN ('K')
                OR :OLD.FB LIKE '%H%'AND :OLD.PKS IN ('K')
    THEN nLM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%H%'AND :NEW.PKS IN ('P')
                   OR :OLD.FB LIKE '%H%'AND :OLD.PKS IN ('P')
    THEN nLM_PRB_UPD:=1;
  ELSE nLM_KTR_UPD:=0; nLM_PRB_UPD:=0;
  END IF;
  
  /* TAM        */
  IF INSERTING AND :NEW.FB LIKE '%T%'AND :NEW.PKS IN ('S')
                OR :OLD.FB LIKE '%T%'AND :OLD.PKS IN ('S')
    THEN nTAM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%T%'AND :NEW.PKS IN ('Z','E')
                   OR :OLD.FB LIKE '%T%'AND :OLD.PKS IN ('Z','E')
    THEN nTAM_PRB_UPD:=1;
  ELSE nTAM_KTR_UPD:=0; nTAM_PRB_UPD:=0;
  END IF;
  
  /* FM        */
  IF INSERTING AND :NEW.FB LIKE '%U%'AND :NEW.PKS IN ('Q')
                OR :OLD.FB LIKE '%U%'AND :OLD.PKS IN ('Q')
    THEN nFM_KTR_UPD:=1;
  ELSIF INSERTING AND :NEW.FB LIKE '%U%'AND :NEW.PKS IN ('N')
                   OR :OLD.FB LIKE '%U%'AND :OLD.PKS IN ('N')
    THEN nFM_PRB_UPD:=1;
  ELSE nFM_KTR_UPD:=0; nFM_PRB_UPD:=0;
  END IF;
  
  BEGIN
    SELECT * INTO recUPD_DATA FROM WKD_D0101_MOBIL WHERE ID=sD0101_ID;
    IF nLM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.LM_KTR THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	END IF;
    IF nLM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.LM_PRB THEN
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.TS_KTR THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	END IF;
    IF nTAM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.TS_PRB THEN
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
	IF nFM_KTR_UPD=1 AND dUPD_DATE>recUPD_DATA.FM_KTR THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	END IF;
    IF nFM_PRB_UPD=1 AND dUPD_DATE>recUPD_DATA.FM_PRB THEN
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    UPDATE WKD_D0101_MOBIL SET ROW = recUPD_DATA WHERE ID = sD0101_ID;

  EXCEPTION WHEN NO_DATA_FOUND THEN
    recUPD_DATA.ID:=sD0101_ID;
    recUPD_DATA.D0101_UPDATE:=TO_DATE('01.01.1900','DD.MM.YYYY');
    recUPD_DATA.LM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.LM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.FM_PRB:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_KTR:=recUPD_DATA.D0101_UPDATE;
    recUPD_DATA.TS_PRB:=recUPD_DATA.D0101_UPDATE;

    IF nLM_KTR_UPD=1 THEN
	  recUPD_DATA.LM_KTR:=dUPD_DATE;
	END IF;
    IF nLM_PRB_UPD=1 THEN
	  recUPD_DATA.LM_PRB:=dUPD_DATE;
	END IF;
	IF nTAM_KTR_UPD=1 THEN
	  recUPD_DATA.TS_KTR:=dUPD_DATE;
	END IF;
    IF nTAM_PRB_UPD=1 THEN
	  recUPD_DATA.TS_PRB:=dUPD_DATE;
	END IF;
	IF nFM_KTR_UPD=1 THEN
	  recUPD_DATA.FM_KTR:=dUPD_DATE;
	END IF;
    IF nFM_PRB_UPD=1 THEN
	  recUPD_DATA.FM_PRB:=dUPD_DATE;
	END IF;
    INSERT INTO WKD_D0101_MOBIL VALUES recUPD_DATA;
  END;

END;
/