/******************************************************************************

SN_LVAX_DMS-SST
Datenbanktabellen

SVN Revisionsinformationen
$Revision: 415 $
$Date: 2015-10-19 10:06:23 +0200 (Mo, 19 Okt 2015) $  

******************************************************************************/
DECLARE 
  vCnt INT;
  vIDX_TBS VARCHAR2(50) := NULL; 
  vUI_TBS VARCHAR2(60) := NULL; 
        
  FUNCTION FieldExists(pTable_name IN VARCHAR2, pColumn_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tab_columns
    WHERE table_name = pTable_name
      AND column_name = pColumn_name;
      
    RETURN vCnt > 0; 
  
  END;

  FUNCTION TableExists(pTable_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tables
    WHERE table_name = pTable_name;
      
    RETURN vCnt > 0; 
  
  END;

  PROCEDURE DropView(pView_name IN VARCHAR2) AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_views
    WHERE view_name = pView_name;
      
    IF vCnt > 0 THEN
      EXECUTE IMMEDIATE 'DROP VIEW ' || pView_name;
    END IF; 
  
  END;
    
  FUNCTION IdxExists(pIndex_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_indexes
    WHERE index_name = pIndex_Name;
      
    RETURN vCnt > 0; 
  
  END;  
  
  PROCEDURE CrtIdx(pSQL IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE pSQL || vIDX_TBS;
  END;  

  FUNCTION ConstraintExists(pConstraint_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_constraints
    WHERE constraint_name = pConstraint_name;
      
    RETURN vCnt > 0; 
  
  END;
   

  PROCEDURE COMMENT_COL(pTab_Col IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON COLUMN ' || pTab_Col || ' IS ''' || pComment || '''';  
  END;
  
  PROCEDURE COMMENT_TAB(pTab IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON TABLE ' || pTab || ' IS ''' || pComment || '''';  
  END;
             
BEGIN

  EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_LENGTH_SEMANTICS = CHAR';

  -- Index-Tablespace merken
  SELECT tablespace_name INTO vIDX_TBS
  FROM user_indexes
  WHERE index_name = 'PK_D0101_ID';
  
  IF vIDX_TBS IS NOT NULL THEN
    vIDX_TBS := ' TABLESPACE ' || vIDX_TBS;
    vUI_TBS := ' USING INDEX ' || vIDX_TBS;
  END IF;

  IF NOT TableExists('DMS_BETRIEB') THEN
    EXECUTE IMMEDIATE q'[   
      CREATE  TABLE DMS_BETRIEB
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           MANDANT NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_DATE DATE NULL ,
           UPD_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           UPD_DATE DATE NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           DEL_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           DEL_DATE DATE NULL ,
           D0101_ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           BETRIEB_FB VARCHAR2(128) NULL ,
           BETRIEB_BEZ VARCHAR2(400) NULL ,
           BETRIEB_NR VARCHAR2(128) NULL ,
           BETRIEB_STR VARCHAR2(128) NULL ,
           BETRIEB_ORT VARCHAR2(128) NULL ,
           BETRIEB_HAUPTVERANTWORTLICHER VARCHAR2(128) NULL ,
           --BETRIEB_ZUL_FLHYG VARCHAR2(4000) NULL ,
           --BETRIEB_HIT_NR VARCHAR2(128) NULL ,
           --BETRIEB_LM_HAUPT_BA VARCHAR2(254) NULL ,
           --BETRIEB_LM_BA VARCHAR2(4000) NULL ,
           --BETRIEB_TS_BA VARCHAR2(4000) NULL ,
           BETRIEB_ANMELDUNG DATE NULL ,
           BETRIEB_ABMELDUNG DATE NULL ,
           BETREIBER VARCHAR2(128) NULL ,
           VERSION_UPDATE NUMBER(10) DEFAULT 0 NOT NULL ,
           VERSION_EXPORT NUMBER(10) DEFAULT -1 NOT NULL ,
           DATE_UPDATE DATE DEFAULT SYSDATE NULL ,
           DATE_EXPORT DATE NULL      ,
           CONSTRAINT PK_DMS_BETRIEB$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE
      )]';


    CrtIdx('CREATE INDEX IDX_DMS_BETRIEB$MANDANT ON DMS_BETRIEB (MANDANT,UPD_DATE,DEL_DATE)');
    CrtIdx('CREATE INDEX IDX_DMS_BETRIEB$D0101_ID ON DMS_BETRIEB (D0101_ID)');

    COMMENT_TAB('DMS_BETRIEB', 'DMS - Betriebsakte');
    
    COMMENT_COL('DMS_BETRIEB.DELETED', '0 - Standard, 1 - inaktiv, 2 - gelscht');
    COMMENT_COL('DMS_BETRIEB.BETRIEB_NR', 'Kennzeichen-Nr.');
    COMMENT_COL('DMS_BETRIEB.BETRIEB_STR', 'Strae, Hausnr. und Zusatz');
    COMMENT_COL('DMS_BETRIEB.BETRIEB_ORT', 'Ort und Ortsteil');
    --COMMENT_COL('DMS_BETRIEB.BETRIEB_ZUL_FLHYG', 'alle d010102.nummer mit Kennzeichen = Z und a010145.code = 0103*, wird erst beim Export aufbereitet');
    --COMMENT_COL('DMS_BETRIEB.BETRIEB_HIT_NR', 'Nr.mit Code 0102, wird erst beim Export aufbereitet');
    --COMMENT_COL('DMS_BETRIEB.BETRIEB_LM_HAUPT_BA', 'angemeldete LM-Hauptbetriebsart, wird erst beim Export aufbereitet');
    --COMMENT_COL('DMS_BETRIEB.BETRIEB_LM_BA', 'angemeldete LM-Nebenbetriebsarten mit Semikolon getrennt, wird erst beim Export aufbereitet');
    --COMMENT_COL('DMS_BETRIEB.BETRIEB_TS_BA', 'angemeldete TS-Betriebsarten mit Semikolon getrennt, wird erst beim Export aufbereitet');
    COMMENT_COL('DMS_BETRIEB.BETREIBER', 'Bez des Betreibers');
    COMMENT_COL('DMS_BETRIEB.VERSION_UPDATE', 'beim Datensatzanlegen standardmig 0, wird mit jedem Aktualisieren inkrementiert, 0 - neuer Datensatz wurde noch nicht gendert');
    COMMENT_COL('DMS_BETRIEB.VERSION_EXPORT', 'Exportversion ist beim Anlegen -1, beim Export wird der  Wert aus VERSION_UPDATE mit bertragen und nach dem erfolgreichen Export ans DMS in VERSION_EXPORT gespeichert, -1 Datensatz wurde noch nicht exportiert');
    COMMENT_COL('DMS_BETRIEB.DATE_UPDATE', 'Datum Aktualisierung VERSION_UPDATE');
    COMMENT_COL('DMS_BETRIEB.DATE_EXPORT', 'Datum Aktualisierung VERSION_EXPORT');

    EXECUTE IMMEDIATE ' 
      ALTER TABLE DMS_BETRIEB ADD
      (
           CONSTRAINT D0101_DMS_BETRIEB$D0101_ID FOREIGN KEY
           ( D0101_ID )
           REFERENCES VIS_D0101
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';

  END IF; -- dms_betrieb
  
  IF NOT TableExists('DMS_DATEN') THEN
    EXECUTE IMMEDIATE q'[
      CREATE  TABLE DMS_DATEN
      ( 
           ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           MANDANT NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_DATE DATE NULL ,
           UPD_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           UPD_DATE DATE NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           DEL_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           DEL_DATE DATE NULL ,
           DMS_BETRIEB_ID VARCHAR2(32) NULL ,
           D0101_ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           D0303_ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           D030301_ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           DOK_OBJEKT_CODE VARCHAR2(128) NULL ,
           DOK_OBJEKT_BEZ VARCHAR2(128) NULL ,
           DOK_BEZ VARCHAR2(128) NULL ,
           DOK_INS_DATE DATE NULL ,
           DOK_FB VARCHAR2(128) NULL ,
           REF_ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           DATUM DATE NULL ,
           ART VARCHAR2(128) NULL ,
           PROBE_NR VARCHAR2(128) NULL ,
           PROBE_BEZ VARCHAR2(128) NULL ,
           UEBERWACHER VARCHAR2(128) NULL ,
           VERSION_UPDATE NUMBER(10) DEFAULT 0 NOT NULL ,
           DATE_UPDATE DATE DEFAULT SYSDATE NULL ,
           VERSION_EXPORT NUMBER(10) DEFAULT -1 NOT NULL ,
           DATE_EXPORT DATE NULL ,
           VERANTWORTLICHER VARCHAR2(2000) NULL      ,
           CONSTRAINT PK_DMS_DATEN PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE
      ) ]'; 
 
    CrtIdx('CREATE INDEX IDX_DMS_DATEN$MANDANT ON DMS_DATEN (MANDANT,UPD_DATE,DEL_DATE)');
    CrtIdx('CREATE INDEX IDX_DMS_DATEN$BETRIEB_ID ON DMS_DATEN (DMS_BETRIEB_ID)');
    CrtIdx('CREATE UNIQUE INDEX IDX_DMS_DATEN$D0303_ID ON DMS_DATEN (D0303_ID,D030301_ID)');
    CrtIdx('CREATE INDEX IDX_DMS_DATEN$D0101_ID ON DMS_DATEN (D0101_ID)');
    CrtIdx('CREATE INDEX IDX_DMS_DATEN$D030301_ID ON DMS_DATEN (D030301_ID)');

    COMMENT_TAB('DMS_DATEN', 'Daten zum Dokument');

    COMMENT_COL('DMS_DATEN.DELETED', '0 - Standard, 1 - inaktiv, 2 - gelscht');
    COMMENT_COL('DMS_DATEN.DOK_BEZ', 'd0303.bez');
    COMMENT_COL('DMS_DATEN.DOK_INS_DATE', 'd0303.ins_date');
    COMMENT_COL('DMS_DATEN.REF_ID', 'd0303.ref_id');
    COMMENT_COL('DMS_DATEN.DATUM', 'D010801.DATUM bei Ref-Table = vis_d010801');
    COMMENT_COL('DMS_DATEN.ART', 'A010801.bez  bei Ref-Table = vis_d010801');
    COMMENT_COL('DMS_DATEN.PROBE_NR', 'd0113.proben_zeichen bei Objekt-Code 1.08.04');
    COMMENT_COL('DMS_DATEN.PROBE_BEZ', 'd0113.bez bei Objekt-Code 1.08.04');
    COMMENT_COL('DMS_DATEN.UEBERWACHER', 'd010200.BEZ (d010801.d010200_id) bei Ref-Table = vis_d010801');
    COMMENT_COL('DMS_DATEN.VERSION_UPDATE', 'beim Datensatzanlegen standardmig 0, wird mit jedem Aktualisieren inkrementiert, 0 - neuer Datensatz wurde noch nicht gendert');
    COMMENT_COL('DMS_DATEN.DATE_UPDATE', 'Datum Aktualisierung VERSION_UPDATE');
    COMMENT_COL('DMS_DATEN.VERSION_EXPORT', 'Exportversion ist beim Anlegen -1, beim Export wird der  Wert aus VERSION_UPDATE mit bertragen und nach dem erfolgreichen Export ans DMS in VERSION_EXPORT gespeichert, -1 Datensatz wurde noch nicht exportiert');
    COMMENT_COL('DMS_DATEN.DATE_EXPORT', 'Datum Aktualisierung VERSION_EXPORT');
    COMMENT_COL('DMS_DATEN.VERANTWORTLICHER', 'D0114.VERANTWORTLICHER nur bei Brgerbeschwerden (Ref-Table = VIS_D0114) ohne Betrieb');

    EXECUTE IMMEDIATE ' 
      ALTER TABLE DMS_DATEN ADD
      (
           CONSTRAINT DMS_BETRIEB_DATEN$BETRIEB_ID FOREIGN KEY
           ( DMS_BETRIEB_ID )
           REFERENCES DMS_BETRIEB
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';

    EXECUTE IMMEDIATE ' 
      ALTER TABLE DMS_DATEN ADD
      (
           CONSTRAINT D0303_DMS_DATEN$D0303_ID FOREIGN KEY
           ( D0303_ID )
           REFERENCES VIS_D0303
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';

    EXECUTE IMMEDIATE ' 
      ALTER TABLE DMS_DATEN ADD
      (
           CONSTRAINT D0101_DMS_DATEN$D0101_ID FOREIGN KEY
           ( D0101_ID )
           REFERENCES VIS_D0101
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';

    EXECUTE IMMEDIATE ' 
      ALTER TABLE DMS_DATEN ADD
      (
           CONSTRAINT D030301_DMS_DATEN$D030301_ID FOREIGN KEY
           ( D030301_ID )
           REFERENCES VIS_D030301
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
 
  END IF; -- dms_daten
END;
/

--------------------------------------------------------------------
-- drop table dms_daten;
-- drop TABLE DMS_BETRIEB;
