/******************************************************************************

Release 1.24.xx

******************************************************************************/
DECLARE 
  vCnt INT;
  vIDX_TBS VARCHAR2(50) := NULL; 
  vUI_TBS VARCHAR2(60) := NULL; 
        
  FUNCTION FieldExists(pTable_name IN VARCHAR2, pColumn_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tab_columns
    WHERE table_name = pTable_name
      AND column_name = pColumn_name;
      
    RETURN vCnt > 0; 
  
  END;

  FUNCTION TableExists(pTable_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tables
    WHERE table_name = pTable_name;
      
    RETURN vCnt > 0; 
  
  END;

  PROCEDURE DropView(pView_name IN VARCHAR2) AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_views
    WHERE view_name = pView_name;
      
    IF vCnt > 0 THEN
      EXECUTE IMMEDIATE 'DROP VIEW ' || pView_name;
    END IF; 
  
  END;
    
  FUNCTION IdxExists(pIndex_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_indexes
    WHERE index_name = pIndex_Name;
      
    RETURN vCnt > 0; 
  
  END;  
  
  PROCEDURE CrtIdx(pSQL IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE pSQL || vIDX_TBS;
  END;  

  FUNCTION ConstraintExists(pConstraint_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_constraints
    WHERE constraint_name = pConstraint_name;
      
    RETURN vCnt > 0; 
  
  END;
   

  PROCEDURE COMMENT_COL(pTab_Col IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON COLUMN ' || pTab_Col || ' IS ''' || pComment || '''';  
  END;
  
  PROCEDURE COMMENT_TAB(pTab IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON TABLE ' || pTab || ' IS ''' || pComment || '''';  
  END;
             
BEGIN

  EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_LENGTH_SEMANTICS = CHAR';

  -- Index-Tablespace merken
  SELECT tablespace_name INTO vIDX_TBS
  FROM user_indexes
  WHERE index_name = 'PK_D0101_ID';
  
  IF vIDX_TBS IS NOT NULL THEN
    vIDX_TBS := ' TABLESPACE ' || vIDX_TBS;
    vUI_TBS := ' USING INDEX ' || vIDX_TBS;
  END IF;
  IF Not TableExists('DEPLOYABLE_RESOURCES') THEN

    EXECUTE IMMEDIATE q'[
      CREATE TABLE DEPLOYABLE_RESOURCES
      ( 
           ID VARCHAR2(32) DEFAULT '-1' NOT NULL ,
           MANDANT NUMBER(10) NULL ,
           CODE VARCHAR2(128) NULL ,
           CODEN NUMBER(10) NULL ,
           BEZ VARCHAR2(128) NULL ,
           BEM VARCHAR2(2000) NULL ,
           INS_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_DATE DATE NULL ,
           UPD_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           UPD_DATE DATE NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           DEL_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           DEL_DATE DATE NULL ,
           FB VARCHAR2(32) DEFAULT '%' NOT NULL ,
           ZB VARCHAR2(32) DEFAULT '%' NOT NULL ,
           RESOURCE_TYPE NUMBER(10) NULL ,
           CLIENT_VERSION VARCHAR2(16) NULL ,
           CUSTOM_FILTER VARCHAR2(64) NULL ,
           TEXT_FILE_REPOSITORY_ID VARCHAR2(32) NULL ,
           ZUGRIFF NUMBER(10) NULL      ,
           CONSTRAINT PK_DEPLOYABLE_RESOURCES PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ]' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_DEPLOYABLE_RESOURCES$01 ON DEPLOYABLE_RESOURCES (MANDANT,UPD_DATE,DEL_DATE)' );
    COMMENT_COL('DEPLOYABLE_RESOURCES.DELETED', '0 - Standard, 1 - inaktiv, 2 - gelscht');
    COMMENT_COL('DEPLOYABLE_RESOURCES.RESOURCE_TYPE', 'Art der Resource: LIZENZ = 1, REPORT= 2, EXTENTSION = 3, KOPF = 4');
    COMMENT_COL('DEPLOYABLE_RESOURCES.CLIENT_VERSION', 'Version des Client fr den die Ressource bestimmt ist');
    COMMENT_COL('DEPLOYABLE_RESOURCES.CUSTOM_FILTER', 'Spezieller Filter');
    COMMENT_COL('DEPLOYABLE_RESOURCES.TEXT_FILE_REPOSITORY_ID', 'ID der eigentlichen Datei aus TEXT_FILE_REPOSITORY');
    COMMENT_COL('DEPLOYABLE_RESOURCES.ZUGRIFF', '3 - BALVI, 2- Admin **, 1 - Admin *');
  END IF;
  
  -- 13.07.2015
  IF NOT FieldExists('DEPLOYABLE_RESOURCES', 'S1711_SID') THEN
    EXECUTE IMMEDIATE '
      ALTER TABLE DEPLOYABLE_RESOURCES ADD
      (
         S1711_SID NUMBER(10) DEFAULT (-1)  NOT NULL,
         
         CONSTRAINT FK_DEPLOYABLE_RESOURCES$S1711 FOREIGN KEY ( S1711_SID )
         REFERENCES VIS_S1711 ( SID ) NOT DEFERRABLE Initially IMMEDIATE            
      )';

    CrtIdx('CREATE INDEX IDX_DEPLOYABLE_RESOURCES$S1711 ON DEPLOYABLE_RESOURCES (S1711_SID)');
      
    Comment_Col('DEPLOYABLE_RESOURCES.S1711_SID', 'Vorlagenverwaltung');
  END IF;
END;
/
